% This program generates draws for historical decomposition
%%% By Christiane Baumeister and James D. Hamilton (Aug 2015)

% To be run after running main_general.m and irf.m, which should have set variables including:
%  psi_invA = [n x n x (hmax+nlags) x nuse] array which includes structural
%     impulse-response matrices
%     dimension 1: variable affected
%     dimension 2: shock whose effect is sought
%     dimension 3: hmax + s component is effect of shock after s periods (s = 0,1,..,hmax-1)
%     dimension 4: indexes the Monte Carlo draw of the parameters
%  note psi_invA is the cumulation of IRFs if kcum = 1
%  YY = (T x n) matrix of observations on dependent variable
%  XX = (T x k) matrix of observations on explanatory variables
%  a_post = [na x (ndraws - nburn)] matrix of draws for elements that go into (n x n) matrix A
%  Bdraw = (n x k x nuse) array of draws for the (n x k) matrix B

% output:
% QH = (n x n x T x nuse) array of components of y attributable to each shock
%     dimension 1: variable affected
%     dimension 2: shock whose effect is sought
%     dimension 3: date of observation
%     dimension 4: indexes the Monte Carlo draw of the parameters

QH = zeros(n,n,T,nuse);

% ===========================================================
% 'Generating draws for historical decomposition'
isim = 0;
while isim < nuse
    isim = isim + 1;
    if (isim/10000) == floor(isim/10000)
        isim
    end

    % ============================================================
    % construct U = (T x n) matrix containing history of structural residuals for this draw
    %A = inv(invA);   % delete this line when no longer in checking mode
    A = setA(a_post(:,isim),kexample);  %uncomment this when no longer
    %  in checking mode
    B = squeeze(Bdraw(:,:,isim));
    U = YY*A' - XX*B';

    for j = 1:n
        % =============================================
        % find historical contribution of structural shock j
        ulast = U(:,j);
        uj = zeros(T,n);
        for hsim = 0:hmax
            uj = uj + ulast*squeeze(psi_invA(:,j,nlags+hsim,isim))';
            ulast = [0; ulast(1:end-1)];
        end
    QH(:,j,:,isim) = uj';
    end
end

% ============================================================
% calculate median and percentiles
index0 = round(0.5*nuse);
index1 = round(0.025*nuse);
index2 = round((1 - 0.025)*nuse);

QH0 = zeros(n,n,T);
QH1 = zeros(n,n,T);
QH2 = zeros(n,n,T);

for i = 1:n
    for j = 1:n
        for t = 1:T
           K = squeeze(QH(i,j,t,:));
           K = sort(K);
           QH0(i,j,t) = K(index0);
           QH1(i,j,t) = K(index1);
           QH2(i,j,t) = K(index2);
        end
    end
end
    
% =========================================================
% Report results
display('Columns report contributions of different shocks for indicated variable')   
varnames2 = char('gap','inflation','fed funds');
for i = 1:n
    display(strcat('median for ',varnames(i,:)))
    format bank
    [time squeeze(QH0(i,:,:))']
    format short
    %['lower 2.5% for ' varnames2(i,:)]
    %squeeze(QH1(i,:,:))'
    %['upper 2.5% for ' varnames2(i,:)]
    %squeeze(QH0(i,:,:))'
end
    
% ==================================================
% display historical decompositions as graphs
for i=1:n
    figure(11+i)
    yy = YY(:,i) - mean(YY(:,i));
          
    for j = 1:n
       subplot(n,1,j)
       temp1=[squeeze(QH0(i,j,:)) squeeze(QH1(i,j,:)) squeeze(QH2(i,j,:))];
       plotx2(temp1,time);
       box on;
       plot(time,yy,'r:','LineWidth',2), hold on, 
       plot(time,temp1(:,1),'b','LineWidth',2), hold on, 
       plot(time,zeros(size(time,1),1),'k:') 
       axis([date_start date_end -5 5])
       temptitle = strcat('Effect of ',shocknames(j),' shocks on ',varnames(i));
       title(temptitle)
    end
end
     

